/*
Purpose: Retrieves Pokémon stats from memory and displays them on the bottom
screen. Shows both Wild Pokémon and Party Pokémon stats. Each can be toggled
on or off with L & R
*/

/*
12a13
> #define __dv (*(vu16*)0x0892fd19)
113c114
< xsprintf(buf, "%02d:%02d", hour, min);
---
> xsprintf(buf, "%04X", __dv);

Original code by shadowofdarkness

Gba Temp Thread
https://gbatemp.net/threads/release-gen-1-pokemon-vc-wild-encounter-dv-ntr-overlay.459869/

v1.1.0U
By Veund & shadowofdarkness

Additions and formatting by AnalogMan

French red offset found thanks to TUK_Pyroman.

Special thanks to greenDarkness555 for heaps of testing of features.
*/

#include "global.h"
#include "ov.h"

FS_archive sdmcArchive = { 0x9, (FS_path){ PATH_EMPTY, 1, (u8*)"" } };
Handle fsUserHandle = 0;
Result APT_CheckNew3DS(bool* out);
void initSharedFunc(void);

#define CALLBACK_OVERLAY (101)

#define TICKS_PER_MSEC (268123.480)

#define WHITE 0xFFFFFF
#define RED   0xFF0000
#define GREEN 0x00FF00
#define BLUE  0x0000FF
#define GOLD  0xD4AF37
#define PURPLE 0x551A8B
#define LBLUE 0xADD8E6

/* Base game offsets

0x89230b8	ENG Red
0x8922d28	ENG Blue
0x8923588	ENG Yellow
0x8a2406c	ENG Gold and Silver
0x8a220f8	ENG Crystal

*/


/* version checks*/
#define __ver (*(vu32*)0x0880014f) /*Game Version Location variable*/
#define __int (*(vu16*)0x002A26D6) /*Game International Version Location variable*/
#define __Yint (*(vu16*)0x002A36D6) /*Game Yellow International Version Location variable*/

/*Pokemon Blue*/
	/*English 0x8922d28 */
#define __bbat  (*(vu8*)0x0892FD7F) /*ENG d057 Pokemon Blue Battle Detection*/
#define __bpar  (*(vu8*)0x0892FCBC) /*ENG cf94 Pokemon Blue Status Screen Detection*/
#define __dvbw  (*(vu16*)0x0892FD19) /*ENG cff1 Pokemon Blue DV location for wild battle*/
#define __dvbp  (*(vu16*)0x0892FCDB) /*ENG cfb3 Pokemon Blue DV location for party*/
#define __expb0 (*(vu8*)0x0892fcd0) /*ENG CFA5*/
#define __expb1 (*(vu8*)0x0892fccf) /*ENG CFA4*/
#define __expb2 (*(vu8*)0x0892fcce) /*ENG CFA3*/
	/*Japanese*/
#define __bbatJ  (*(vu8*)0x088AFBD4) /*JPN d057 Pokemon Blue Battle Detection*/
#define __bparJ  (*(vu8*)0x088AFB1B) /*JPN cf94 Pokemon Blue Status Screen Detection*/
#define __dvbwJ  (*(vu16*)0x088afb78) /*JPN cff1 Pokemon Blue DV location for wild battle*/
#define __dvbpJ  (*(vu16*)0x088AFB3A) /*JPN cfb3 Pokemon Blue DV location for party*/
#define __expb0J (*(vu8*)0x088AFB2F) /*JPN CFA5*/
#define __expb1J (*(vu8*)0x088AFB2E) /*JPN CFA4*/
#define __expb2J (*(vu8*)0x088AFB2D) /*JPN CFA3*/
	/*Spanish*/
#define __bbatS  (*(vu8*)0x08915F32) /*SPA d057 Pokemon Blue Battle Detection*/
#define __bparS  (*(vu8*)0x0892FF1D) /*SPA cf94 Pokemon Blue Status Screen Detection*/
#define __dvbwS  (*(vu16*)0x0892ff7a) /*SPA cff1 Pokemon Blue DV location for wild battle*/
#define __dvbpS  (*(vu16*)0x0892FF3C) /*SPA cfb3 Pokemon Blue DV location for party*/
#define __expb0S (*(vu8*)0x0892FF21) /*SPA CFA5*/
#define __expb1S (*(vu8*)0x0892FF20) /*SPA CFA4*/
#define __expb2S (*(vu8*)0x0892FF1F) /*SPA CFA3*/
	/*French*/
#define __bbatF  (*(vu8*)0x0892FDB4) /*FRE d057 Pokemon Blue Battle Detection*/
#define __bparF  (*(vu8*)0x0892FCF1) /*FRE cf94 Pokemon Blue Status Screen Detection*/
#define __dvbwF  (*(vu16*)0x0892fd4e) /*FRE cff1 Pokemon Blue DV location for wild battle*/
#define __dvbpF  (*(vu16*)0x0892FD10) /*FRE cfb3 Pokemon Blue DV location for party*/
#define __expb0F (*(vu8*)0x0892FD05) /*FRE CFA5*/
#define __expb1F (*(vu8*)0x0892FD04) /*FRE CFA4*/
#define __expb2F (*(vu8*)0x0892FD03) /*FRE CFA3*/
	/*German*/
#define __bbatG  (*(vu8*)0x0892FDB4) /*GER d057 Pokemon Blue Battle Detection*/
#define __bparG  (*(vu8*)0x0892FCF1) /*GER cf94 Pokemon Blue Status Screen Detection*/
#define __dvbwG  (*(vu16*)0x0892fd4e) /*GER cff1 Pokemon Blue DV location for wild battle*/
#define __dvbpG  (*(vu16*)0x0892FD10) /*GER cfb3 Pokemon Blue DV location for party*/
#define __expb0G (*(vu8*)0x0892FD05) /*GER CFA5*/
#define __expb1G (*(vu8*)0x0892FD04) /*GER CFA4*/
#define __expb2G (*(vu8*)0x0892FD03) /*GER CFA3*/
	/*Italian*/
#define __bbatI  (*(vu8*)0x0893012C) /*ITL d057 Pokemon Blue Battle Detection*/
#define __bparI  (*(vu8*)0x08930069) /*ITL cf94 Pokemon Blue Status Screen Detection*/
#define __dvbwI  (*(vu16*)0x089300c6) /*ITL cff1 Pokemon Blue DV location for wild battle*/
#define __dvbpI  (*(vu16*)0x08930088) /*ITL cfb3 Pokemon Blue DV location for party*/
#define __expb0I (*(vu8*)0x0893007D) /*ITL CFA5*/
#define __expb1I (*(vu8*)0x0893007C) /*ITL CFA4*/
#define __expb2I (*(vu8*)0x0893007B) /*ITL CFA3*/

/*Pokemon Red*/
	/*English*/
#define __rbat  (*(vu8*)0x0893010F) /*ENG d057 Pokemon Red Battle Detection*/
#define __rpar  (*(vu8*)0x0893004C) /*ENG cf94 Pokemon Red Status Screen Detection*/
#define __dvrw  (*(vu16*)0x089300A9) /*ENG cff1 Pokemon Red DV location for wild battle*/
#define __dvrp  (*(vu16*)0x0893006B) /*ENG cfb3 Pokemon Red DV location for party*/
#define __expr0 (*(vu8*)0x08930060) /*ENG CFA5*/
#define __expr1 (*(vu8*)0x0893005f) /*ENG CFA4*/
#define __expr2 (*(vu8*)0x0893005e) /*ENG CFA3*/
	/*Japanese*/
#define __rbatJ  (*(vu8*)0x088AFB8C) /*JPN d057 Pokemon Red Battle Detection*/
#define __rparJ  (*(vu8*)0x088AFAD3) /*JPN cf94 Pokemon Red Status Screen Detection*/
#define __dvrwJ  (*(vu16*)0x088afb30) /*JPN cff1 Pokemon Red DV location for wild battle*/
#define __dvrpJ  (*(vu16*)0x088AFAF2) /*JPN cfb3 Pokemon Red DV location for party*/
#define __expr0J (*(vu8*)0x088AFAE7) /*JPN CFA5*/
#define __expr1J (*(vu8*)0x088AFAE6) /*JPN CFA4*/
#define __expr2J (*(vu8*)0x088AFAE5) /*JPN CFA3*/
	/*Spanish*/
#define __rbatS  (*(vu8*)0x0893012C) /*SPA d057 Pokemon Red Battle Detection*/
#define __rparS  (*(vu8*)0x08930069) /*SPA cf94 Pokemon Red Status Screen Detection*/
#define __dvrwS  (*(vu16*)0x089300C6) /*SPA cff1 Pokemon Red DV location for wild battle*/
#define __dvrpS  (*(vu16*)0x08930088) /*SPA cfb3 Pokemon Red DV location for party*/
#define __expr0S (*(vu8*)0x0893007D) /*SPA CFA5*/
#define __expr1S (*(vu8*)0x0893007C) /*SPA CFA4*/
#define __expr2S (*(vu8*)0x0893007B) /*SPA CFA3*/
	/*French*/
#define __rbatF  (*(vu8*)0x0892FF18) /*FRE d057 Pokemon Red Battle Detection*/
#define __rparF  (*(vu8*)0x0892FE55) /*FRE cf94 Pokemon Red Status Screen Detection*/
#define __dvrwF  (*(vu16*)0x0892FEB2) /*FRE cff1 Pokemon Red DV location for wild battle*/
#define __dvrpF  (*(vu16*)0x0892FE74) /*FRE cfb3 Pokemon Red DV location for party*/
#define __expr0F (*(vu8*)0x0892FE69) /*FRE CFA5*/
#define __expr1F (*(vu8*)0x0892FE68) /*FRE CFA4*/
#define __expr2F (*(vu8*)0x0892FE67) /*FRE CFA3*/
	/*German*/
#define __rbatG  (*(vu8*)0x0892FDB4) /*GER d057 Pokemon Red Battle Detection*/
#define __rparG  (*(vu8*)0x0892FCF1) /*GER cf94 Pokemon Red Status Screen Detection*/
#define __dvrwG  (*(vu16*)0x0892fd4e) /*GER cff1 Pokemon Red DV location for wild battle*/
#define __dvrpG  (*(vu16*)0x0892FD10) /*GER cfb3 Pokemon Red DV location for party*/
#define __expr0G (*(vu8*)0x0892FD05) /*GER CFA5*/
#define __expr1G (*(vu8*)0x0892FD04) /*GER CFA4*/
#define __expr2G (*(vu8*)0x0892FD03) /*GER CFA3*/
	/*Italian*/
#define __rbatI  (*(vu8*)0x0892FFE0) /*ITL d057 Pokemon Red Battle Detection*/
#define __rparI  (*(vu8*)0x0892FF1D) /*ITL cf94 Pokemon Red Status Screen Detection*/
#define __dvrwI  (*(vu16*)0x0892ff7a) /*ITL cff1 Pokemon Red DV location for wild battle*/
#define __dvrpI  (*(vu16*)0x0892FF3C) /*ITL cfb3 Pokemon Red DV location for party*/
#define __expr0I (*(vu8*)0x0892FF31) /*ITL CFA5*/
#define __expr1I (*(vu8*)0x0892FF30) /*ITL CFA4*/
#define __expr2I (*(vu8*)0x0892FF2F) /*ITL CFA3*/

/*Pokemon Yellow*/
	/*English*/
#define __ybat  (*(vu8*)0x089305DE) /*ENG d056 Pokemon Yellow Battle Detection*/
#define __ypar  (*(vu8*)0x0893051B) /*ENG cf93 Pokemon Yellow Status Screen Detection*/
#define __dvyw  (*(vu16*)0x08930578) /*ENG cff0 Pokemon Yellow DV location for wild battle*/
#define __dvyp  (*(vu16*)0x0893053A) /*ENG cfb2 Pokemon Yellow DV location for party*/
#define __expy0 (*(vu8*)0x0893052f) /*ENG CFA5*/
#define __expy1 (*(vu8*)0x0893052e) /*ENG CFA4*/
#define __expy2 (*(vu8*)0x0893052d) /*ENG CFA3*/
	/*Japanese*/
#define __ybatJ  (*(vu8*)0x0892FF58) /*JPN d056 Pokemon Yellow Battle Detection*/
#define __yparJ  (*(vu8*)0x0892FE9F) /*JPN cf93 Pokemon Yellow Status Screen Detection*/
#define __dvywJ  (*(vu16*)0x0892FEFC) /*JPN cff0 Pokemon Yellow DV location for wild battle*/
#define __dvypJ  (*(vu16*)0x0892FEBE) /*JPN cfb2 Pokemon Yellow DV location for party*/
#define __expy0J (*(vu8*)0x0892FEB3) /*JPN CFA5*/
#define __expy1J (*(vu8*)0x0892FEB2) /*JPN CFA4*/
#define __expy2J (*(vu8*)0x0892FEB1) /*JPN CFA3*/
	/*Spanish*/
#define __ybatS  (*(vu8*)0x089308F3) /*SPA d056 Pokemon Yellow Battle Detection*/
#define __yparS  (*(vu8*)0x08930830) /*SPA cf93 Pokemon Yellow Status Screen Detection*/
#define __dvywS  (*(vu16*)0x0893088d) /*SPA cff0 Pokemon Yellow DV location for wild battle*/
#define __dvypS  (*(vu16*)0x0893084F) /*SPA cfb2 Pokemon Yellow DV location for party*/
#define __expy0S (*(vu8*)0x08930844) /*SPA CFA5*/
#define __expy1S (*(vu8*)0x08930843) /*SPA CFA4*/
#define __expy2S (*(vu8*)0x08930842) /*SPA CFA3*/
	/*French*/
#define __ybatF  (*(vu8*)0x08930917) /*FRE d056 Pokemon Yellow Battle Detection*/
#define __yparF  (*(vu8*)0x08930854) /*FRE cf93 Pokemon Yellow Status Screen Detection*/
#define __dvywF  (*(vu16*)0x089308b1) /*FRE cff0 Pokemon Yellow DV location for wild battle*/
#define __dvypF  (*(vu16*)0x08930873) /*FRE cfb2 Pokemon Yellow DV location for party*/
#define __expy0F (*(vu8*)0x08930868) /*FRE CFA5*/
#define __expy1F (*(vu8*)0x08930867) /*FRE CFA4*/
#define __expy2F (*(vu8*)0x08930866) /*FRE CFA3*/
	/*German*/
#define __ybatG  (*(vu8*)0x089308E7) /*GER d056 Pokemon Yellow Battle Detection*/
#define __yparG  (*(vu8*)0x08930824) /*GER cf93 Pokemon Yellow Status Screen Detection*/
#define __dvywG  (*(vu16*)0x08930881) /*GER cff0 Pokemon Yellow DV location for wild battle*/
#define __dvypG  (*(vu16*)0x08930843) /*GER cfb2 Pokemon Yellow DV location for party*/
#define __expy0G (*(vu8*)0x08930838) /*GER CFA5*/
#define __expy1G (*(vu8*)0x08930837) /*GER CFA4*/
#define __expy2G (*(vu8*)0x08930836) /*GER CFA3*/
	/*Italian*/
#define __ybatI  (*(vu8*)0x089308F3) /*ITL d056 Pokemon Yellow Battle Detection*/
#define __yparI  (*(vu8*)0x08930830) /*ITL cf93 Pokemon Yellow Status Screen Detection*/
#define __dvywI  (*(vu16*)0x0893088d) /*ITL cff0 Pokemon Yellow DV location for wild battle*/
#define __dvypI  (*(vu16*)0x0893084F) /*ITL cfb2 Pokemon Yellow DV location for party*/
#define __expy0I (*(vu8*)0x08930844) /*ITL CFA5*/
#define __expy1I (*(vu8*)0x08930843) /*ITL CFA4*/
#define __expy2I (*(vu8*)0x08930842) /*ITL CFA3*/



/*Pokemon Green*/
#define __gbat  (*(vu8*)0x088AFC1C) /*JPN d057 Pokemon Green Battle Detection*/ 
#define __gpar  (*(vu8*)0x088AFB63) /*JPN cf94 Pokemon Green Status Screen Detection*/
#define __dvgw  (*(vu16*)0x088afbc0) /*JPN cff1 Pokemon Green DV location for wild battle*/
#define __dvgp  (*(vu16*)0x088AFB82) /*JPN cfb3 Pokemon Green DV location for party*/
#define __expg0 (*(vu8*)0x088AFB77) /*JPN CFA5*/
#define __expg1 (*(vu8*)0x088AFB76) /*JPN CFA4*/
#define __expg2 (*(vu8*)0x088AFB75) /*JPN CFA3*/


/*Pokemon Gold/Silver*/ 

#define __dvstest  (*(vu16*)0x08A3108C) /*ENG Pokemon Gold/Silver DV location current viewed TEST*/

#define __playsM  (*(vu16*)0x08A31A6C) /*ENG da00 player loaction*/
#define __sbat  (*(vu8*)0x08A31182) /*ENG d116 Pokemon Gold/Silver Battle Detection*/
#define __spar  (*(vu8*)0x08A30F44) /*ENG ced8 Pokemon Gold/Silver Party List Detection*/
#define __dvsw  (*(vu16*)0x08A31161) /*ENG d0f5 Pokemon Gold/Silver DV location for wild battle*/
//#define __dvsp1  (*(vu16*)0x08A31AAB) /*ENG da3f Pokemon Gold/Silver DV location for 1st party*/
//#define __dvsp2  (*(vu16*)0x08A31ADB) /*ENG da6f Pokemon Gold/Silver DV location for 2nd party*/
//#define __dvsp3  (*(vu16*)0x08A31B0B) /*ENG da9f Pokemon Gold/Silver DV location for 3rd party*/
//#define __dvsp4  (*(vu16*)0x08A31B3B) /*ENG dacf Pokemon Gold/Silver DV location for 4th party*/
//#define __dvsp5  (*(vu16*)0x08A31B6B) /*ENG daff Pokemon Gold/Silver DV location for 5th party*/
//#define __dvsp6  (*(vu16*)0x08A31B9B) /*ENG db2f Pokemon Gold/Silver DV location for 6th party*/
#define __dvse  (*(vu16*)0x08A31D47) /*ENG dcdb Pokemon Gold/Silver DV location for daycare egg*/
#define __segg  (*(vu8*)0x8A318FF) /*ENG d893 Pokemon Gold/Silver man outside at daycare*/
#define __roamsr  (*(vu16*)0x08A31D8B) /*ENG dd1f Pokemon Gold/Silver Raikou*/
#define __roamsMr  (*(vu16*)0x08A31D88) /*ENG dd1c Pokemon Gold/Silver Raikou*/
#define __roamse  (*(vu16*)0x08A31D92) /*ENG dd26 Pokemon Gold/Silver Entei*/
#define __roamsMe  (*(vu16*)0x08A31D8F) /*ENG dd23 Pokemon Gold/Silver Entei*/
#define __roamss  (*(vu16*)0x08A31D99) /*ENG dd2d Pokemon Gold/Silver Suicune*/
#define __roamsMs  (*(vu16*)0x08A31D96) /*ENG dd2a Pokemon Gold/Silver Suicune*/
#define __ibats  (*(vu8*)0x08A3115C) /*ENG d0f0 Pokemon Gold/Silver wild pokemon Item*/
#define __menus (*(vu8*)0x08A30F4C) /*ENG cee0 Pokemon Gold/Silver current menu selection*/
#define __exp2 (*(vu8*)0x08A3107f)/*ENG da32 party pokemon 1 exp Gold/Silver*/
#define __exp1 (*(vu8*)0x08A31080)/*ENG da33 party pokemon 1 exp Gold/Silver*/
#define __exp0 (*(vu8*)0x08A31081)/*ENG da34 party pokemon 1 exp Gold/Silver*/
//#define __exp21 (*(vu8*)0x08A31ACE)/*ENG da62 party pokemon 2 exp Gold/Silver*/
//#define __exp11 (*(vu8*)0x08A31ACF)/*ENG da63 party pokemon 2 exp Gold/Silver*/
//#define __exp01 (*(vu8*)0x08A31AD0)/*ENG da64 party pokemon 2 exp Gold/Silver*/
//#define __exp22 (*(vu8*)0x08A31AFE)/*ENG da92 party pokemon 3 exp Gold/Silver*/
//#define __exp12 (*(vu8*)0x08A31AFF)/*ENG da93 party pokemon 3 exp Gold/Silver*/
//#define __exp02 (*(vu8*)0x08A31B00)/*ENG da94 party pokemon 3 exp Gold/Silver*/
//#define __exp23 (*(vu8*)0x08A31B2E)/*ENG dac2 party pokemon 4 exp Gold/Silver*/
//#define __exp13 (*(vu8*)0x08A31B2F)/*ENG dac3 party pokemon 4 exp Gold/Silver*/
//#define __exp03 (*(vu8*)0x08A31B30)/*ENG dac4 party pokemon 4 exp Gold/Silver*/
//#define __exp24 (*(vu8*)0x08A31B5E)/*ENG daf2 party pokemon 5 exp Gold/Silver*/
//#define __exp14 (*(vu8*)0x08A31B5F)/*ENG daf3 party pokemon 5 exp Gold/Silver*/
//#define __exp04 (*(vu8*)0x08A31B60)/*ENG daf4 party pokemon 5 exp Gold/Silver*/
//#define __exp25 (*(vu8*)0x08A31B8E)/*ENG db22 party pokemon 6 exp Gold/Silver*/
//#define __exp15 (*(vu8*)0x08A31B8F)/*ENG db23 party pokemon 6 exp Gold/Silver*/
//#define __exp05 (*(vu8*)0x08A31B90)/*ENG db24 party pokemon 6 exp Gold/Silver*/

/*Pokemon Crystal*/
        /*English*/
#define __playcM  (*(vu16*)0x08A2FDAD) /*ENG dcb5? player loaction*/
#define __cbat  (*(vu8*)0x08A2F325) /*ENG d22d Pokemon Crystal Battle Detection*/
#define __cpar  (*(vu8*)0x08A2F099) /*ENG cfa1 Pokemon Crystal Party List Detection*/
#define __dvcw  (*(vu16*)0x08A2F304) /*ENG d20c Pokemon Crystal DV location for wild battle*/
#define __dvcp1  (*(vu16*)0x08A2FDEC) /*ENG dcf4 Pokemon Crystal DV location for 1st party*/
#define __dvcp2  (*(vu16*)0x08A2FE1C) /*ENG dcf4 Pokemon Crystal DV location for 2nd party*/
#define __dvcp3  (*(vu16*)0x08A2FE4C) /*ENG dcf4 Pokemon Crystal DV location for 3rd party*/
#define __dvcp4  (*(vu16*)0x08A2FE7C) /*ENG dcf4 Pokemon Crystal DV location for 4th party*/
#define __dvcp5  (*(vu16*)0x08A2FEAC) /*ENG dcf4 Pokemon Crystal DV location for 5th party*/
#define __dvcp6  (*(vu16*)0x08A2FEDC) /*ENG dcf4 Pokemon Crystal DV location for 6th party*/
#define __dvce  (*(vu16*)0x08A30088) /*ENG df90 Pokemon Crystal DV location for daycare egg*/
#define __cegg  (*(vu8*)0x8A2FC46) /*ENG db4e Pokemon Crystal man outside at daycare*/
#define __roamcr  (*(vu16*)0x08A300CC) /*ENG dfd4 Pokemon Crystal Raikou*/
#define __roamcMr  (*(vu16*)0x08A300C9) /*ENG dfd1 Pokemon Crystal Raikou*/
#define __roamce  (*(vu16*)0x08A300D3) /*ENG dfdb Pokemon Crystal Entei*/
#define __roamcMe  (*(vu16*)0x08A300D0) /*ENG dfd8 Pokemon Crystal Entei*/
#define __ibatc  (*(vu8*)0x08A2F2FF) /*ENG d207 Pokemon Crystal wild pokemon Item*/
#define __menuc (*(vu8*)0x08A2F0A1) /*ENG cfa9 Pokemon Crystal current menu selection*/
#define __cexp2 (*(vu8*)0x08A2FDDF)/*ENG dce7 party pokemon 1 exp Crystal*/
#define __cexp1 (*(vu8*)0x08A2FDE0)/*ENG dce8 party pokemon 1 exp Crystal*/
#define __cexp0 (*(vu8*)0x08A2FDE1)/*ENG dce9 party pokemon 1 exp Crystal*/
#define __cexp21 (*(vu8*)0x08A2FE0F)/*ENG dd17 party pokemon 2 exp Crystal*/
#define __cexp11 (*(vu8*)0x08A2FE10)/*ENG dd18 party pokemon 2 exp Crystal*/
#define __cexp01 (*(vu8*)0x08A2FE11)/*ENG dd19 party pokemon 2 exp Crystal*/
#define __cexp22 (*(vu8*)0x08A2FE3F)/*ENG dd47 party pokemon 3 exp Crystal*/
#define __cexp12 (*(vu8*)0x08A2FE40)/*ENG dd48 party pokemon 3 exp Crystal*/
#define __cexp02 (*(vu8*)0x08A2FE41)/*ENG dd49 party pokemon 3 exp Crystal*/
#define __cexp23 (*(vu8*)0x08A2FE6F)/*ENG dd77 party pokemon 4 exp Crystal*/
#define __cexp13 (*(vu8*)0x08A2FE70)/*ENG dd78 party pokemon 4 exp Crystal*/
#define __cexp03 (*(vu8*)0x08A2FE71)/*ENG dd79 party pokemon 4 exp Crystal*/
#define __cexp24 (*(vu8*)0x08A2FE9F)/*ENG dda7 party pokemon 5 exp Crystal*/
#define __cexp14 (*(vu8*)0x08A2FEA0)/*ENG dda8 party pokemon 5 exp Crystal*/
#define __cexp04 (*(vu8*)0x08A2FEA1)/*ENG dda9 party pokemon 5 exp Crystal*/
#define __cexp25 (*(vu8*)0x08A2FECF)/*ENG ddd7 party pokemon 6 exp Crystal*/
#define __cexp15 (*(vu8*)0x08A2FED0)/*ENG ddd8 party pokemon 6 exp Crystal*/
#define __cexp05 (*(vu8*)0x08A2FED1)/*ENG ddd9 party pokemon 6 exp Crystal*/
        /*Spanish 0x8a21ef0*/
#define __playcMS  (*(vu16*)0x08A2FBA5) /*SPA dcb5? player loaction*/
#define __cbatS  (*(vu8*)0x08A2F11D) /*SPA d22d Pokemon Crystal Battle Detection*/
#define __cparS  (*(vu8*)0x08A2EE91) /*SPA cfa1 Pokemon Crystal Party List Detection*/
#define __dvcwS  (*(vu16*)0x08A2F0FC) /*SPA d20c Pokemon Crystal DV location for wild battle*/
#define __dvcp1S  (*(vu16*)0x08A2FBE4) /*SPA dcf4 Pokemon Crystal DV location for 1st party*/
#define __dvcp2S  (*(vu16*)0x08A2FC14) /*SPA dcf4 Pokemon Crystal DV location for 2nd party*/
#define __dvcp3S  (*(vu16*)0x08A2FC44) /*SPA dcf4 Pokemon Crystal DV location for 3rd party*/
#define __dvcp4S  (*(vu16*)0x08A2FC74) /*SPA dcf4 Pokemon Crystal DV location for 4th party*/
#define __dvcp5S  (*(vu16*)0x08A2FCA4) /*SPA dcf4 Pokemon Crystal DV location for 5th party*/
#define __dvcp6S  (*(vu16*)0x08A2FCD4) /*SPA dcf4 Pokemon Crystal DV location for 6th party*/
#define __dvceS  (*(vu16*)0x08A2FE80) /*SPA df90 Pokemon Crystal DV location for daycare egg*/
#define __ceggS  (*(vu8*)0x8A2FA3E) /*SPA db4e Pokemon Crystal man outside at daycare*/
#define __roamcrS  (*(vu16*)0x08A2FEC4) /*SPA dfd4 Pokemon Crystal Raikou*/
#define __roamcMrS  (*(vu16*)0x08A2FEC1) /*SPA dfd1 Pokemon Crystal Raikou*/
#define __roamceS  (*(vu16*)0x08A2FECB) /*SPA dfdb Pokemon Crystal Entei*/
#define __roamcMeS  (*(vu16*)0x08A2FEC8) /*SPA dfd8 Pokemon Crystal Entei*/
#define __ibatcS  (*(vu8*)0x08A2F0F7) /*SPA d207 Pokemon Crystal wild pokemon Item*/
#define __menucS (*(vu8*)0x08A2EE99) /*SPA cfa9 Pokemon Crystal current menu selection*/
#define __cexp2S (*(vu8*)0x08A2FBD7)/*SPA dce7 party pokemon 1 exp Crystal*/
#define __cexp1S (*(vu8*)0x08A2FBD8)/*SPA dce8 party pokemon 1 exp Crystal*/
#define __cexp0S (*(vu8*)0x08A2FBD9)/*SPA dce9 party pokemon 1 exp Crystal*/
#define __cexp21S (*(vu8*)0x08A2FC07)/*SPA dd17 party pokemon 2 exp Crystal*/
#define __cexp11S (*(vu8*)0x08A2FC08)/*SPA dd18 party pokemon 2 exp Crystal*/
#define __cexp01S (*(vu8*)0x08A2FC09)/*SPA dd19 party pokemon 2 exp Crystal*/
#define __cexp22S (*(vu8*)0x08A2FC37)/*SPA dd47 party pokemon 3 exp Crystal*/
#define __cexp12S (*(vu8*)0x08A2FC38)/*SPA dd48 party pokemon 3 exp Crystal*/
#define __cexp02S (*(vu8*)0x08A2FC39)/*SPA dd49 party pokemon 3 exp Crystal*/
#define __cexp23S (*(vu8*)0x08A2FC67)/*SPA dd77 party pokemon 4 exp Crystal*/
#define __cexp13S (*(vu8*)0x08A2FC68)/*SPA dd78 party pokemon 4 exp Crystal*/
#define __cexp03S (*(vu8*)0x08A2FC69)/*SPA dd79 party pokemon 4 exp Crystal*/
#define __cexp24S (*(vu8*)0x08A2FC97)/*SPA dda7 party pokemon 5 exp Crystal*/
#define __cexp14S (*(vu8*)0x08A2FC98)/*SPA dda8 party pokemon 5 exp Crystal*/
#define __cexp04S (*(vu8*)0x08A2FC99)/*SPA dda9 party pokemon 5 exp Crystal*/
#define __cexp25S (*(vu8*)0x08A2FCC7)/*SPA ddd7 party pokemon 6 exp Crystal*/
#define __cexp15S (*(vu8*)0x08A2FCC8)/*SPA ddd8 party pokemon 6 exp Crystal*/
#define __cexp05S (*(vu8*)0x08A2FCC9)/*SPA ddd9 party pokemon 6 exp Crystal*/

char hiddenName[][16] = {"Fighting", "Flying", "Poison", "Ground", "Rock", "Bug", "Ghost", "Steel", "Fire", "Water", "Grass", "Electric", "Psychic", "Ice", "Dragon", "Dark"}; /*Hidden Power Types*/
int hiddenColour[16] = {0xC9656F, 0x71A5FB, 0x9787B8, 0xD3A35A, 0x928167, 0x95AD3F, 0x4D5F9D, 0xC0C0C0, 0xEE0B0B, 0x4045FF, 0x4E9131, 0xFBF259, 0x8230B8, 0x82FFE6, 0x3540BD, 0x4A4A4A }; /*Type colours extracted from a crystal Sprite map *in folder* */
/*							0		 1		2			3		4			5		6			7		8			9		10			11		12			13		14		  15			helps with hidden power debug		*/
char natruegne7[][25] = {"Hardy", "Lonely", "Brave", "Adamant", "Naughty", "Bold", "Docile", "Relaxed", "Impish", "Lax", "Timid", "Hasty", "Serious", "Jolly", "Naive", "Modest", "Mild", "Quiet", "Bashful", "Rash", "Calm", "Gentle", "Sassy", "Careful", "Quirky"}; /*Nature Array*/
int rCol[3] = {RED, GOLD, LBLUE};

u32 IoBasePad;
bool togglePressed = false,
     gen1 = false,
     gen2 = false;

/* Initialize display of overlay boxes to off */
bool partyOn = false,
     wildOn = false,
     eggOn = false,
     roamOn = false;

unsigned int battle = 0,
	     egga = 0,
             party = 0,
	     dvr = 0,
	     menu = 1,
	     pdv1 = 0,
             pdv2 = 0,
             pdv3 = 0,
             pdv4 = 0,
             pdv5 = 0,
             pdv6 = 0,
	     raikou = 0,
	     entei = 0,
	     suicune = 0,
	     crystal = 0;
unsigned int expd0 = 0;
unsigned int expd1 = 0;
unsigned int expd2 = 0;
unsigned int expd01 = 0;
unsigned int expd11 = 0;
unsigned int expd21 = 0;
unsigned int expd02 = 0;
unsigned int expd12 = 0;
unsigned int expd22 = 0;
unsigned int expd03 = 0;
unsigned int expd13 = 0;
unsigned int expd23 = 0;
unsigned int expd04 = 0;
unsigned int expd14 = 0;
unsigned int expd24 = 0;
unsigned int expd05 = 0;
unsigned int expd15 = 0;
unsigned int expd25 = 0;
unsigned int expD2 = 0;
unsigned int expD1 = 0;
unsigned int expD0 = 0;
unsigned int roamMe = 0;
unsigned int roamMr = 0;
unsigned int roamMs = 0;
unsigned int playM = 0;
unsigned int Rcol = 0;
unsigned int Auto = 0;

			
			
unsigned int ibat = 0;


 /*
 DV Calculator
 dv: raw hex value of stats to be calculated
 x: X position on screen box starts at in pixels
 y: Y position on screen box starts at in pixels
 w: width of box in pixels
 h: height of box in pixels
 */
void dvCalc(unsigned int dv, int x, int y, int w, int h)
{
    char buf[60];

    /* Break DV value up into individual stats and calculate HP stat */
    unsigned int speed = (dv >> 0xC) & 0xF,
                 special = (dv >> 0x8) & 0xF,
                 attack = (dv >> 0x4) & 0xF,
                 defense = dv & 0xF,
                 hp = ((attack & 0x1) << 3) + ((defense & 0x1) << 2) + ((speed & 0x1) << 1) + (special & 0x1);
	unsigned int hidden = ((attack & 3) << 2) + (defense & 3);
		
	


    /* Draw transparent box with variable position and size */
    OvDrawTranspartBlackRect(x, y, w, h, 1);

    /* Write HP line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "HP:", hp);
    OvDrawString(buf, x+1, y+14, (hp == 15) ? GREEN : (hp == 0) ? RED : WHITE);

    /* Write Attack line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "Attack:", attack);
    OvDrawString(buf, x+1, y+25, (attack == 15) ? GREEN : (attack == 0) ? RED : WHITE);

    /* Write Defense line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "Defense:", defense);
    OvDrawString(buf, x+1, y+36, (defense == 15) ? GREEN : (defense == 0) ? RED : WHITE);

    /* Write Speed line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "Speed:", speed);
    OvDrawString(buf, x+1, y+47, (speed == 15) ? GREEN : (speed == 0) ? RED : WHITE);

    /* Write Special line and color code according to value and a position relative to box */
    xsprintf(buf, "%-12s %2d", "Special:", special);
    OvDrawString(buf, x+1, y+58, (special == 15) ? GREEN : (special == 0) ? RED : WHITE);
	
	
	/* Write Hidden Power type line and color code according to value and a position relative to box */
	xsprintf(buf,"HP:");
	OvDrawString(buf, x+1, y+87, WHITE); 
	xsprintf(buf, hiddenName[hidden]);
	OvDrawString(buf, x+22, y+87, hiddenColour[hidden]); 


    /*
    Compares DV value to known values that result in a shiny
    when transferred and outputs a position relative string
    if a match is found
    */
    if ((dv & 0xFF2F) == 0xAA2A) {
        xsprintf(buf, "Shiny");
        OvDrawString(buf, x+1, y+74, GOLD);
    } else {
        xsprintf(buf, "Not Shiny");
        OvDrawString(buf, x+1, y+74, WHITE);
    }
}

/*
Determines which game is loaded and assigns stat value from appropriate address.
Checks if overlays are active and calls the ones that are, then writes title of
overlay inside of box.
*/
void drawWidget(void)
{
    unsigned int dvw, dvp, dve;

    switch(__ver) {
        case 0x00000045: // Blue	
			switch(__int) {
				case 0x3836: //English
					dvw = __dvbw;
					dvp = __dvbp;
					battle = __bbat;
					party = __bpar;
					gen1 = true;
					expD1 = __expb1;
					expD2 = __expb2;
					expD0 = __expb0;
					break;
				case 0x3236: //Japanese
					dvw = __dvbwJ;
					dvp = __dvbpJ;
					battle = __bbatJ;
					party = __bparJ;
					gen1 = true;
					expD1 = __expb1J;
					expD2 = __expb2J;
					expD0 = __expb0J;
					break;
				case 0x3135: //Spanish
					dvw = __dvbwS;
					dvp = __dvbpS;
					battle = __bbatS;
					party = __bparS;
					gen1 = true;
					expD1 = __expb1S;
					expD2 = __expb2S;
					expD0 = __expb0S;
					break;
				case 0x3133: //French
					dvw = __dvbwF;
					dvp = __dvbpF;
					battle = __bbatF;
					party = __bparF;
					gen1 = true;
					expD1 = __expb1F;
					expD2 = __expb2F;
					expD0 = __expb0F;
					break;
				case 0x3633: //German
					dvw = __dvbwG;
					dvp = __dvbpG;
					battle = __bbatG;
					party = __bparG;
					gen1 = true;
					expD1 = __expb1G;
					expD2 = __expb2G;
					expD0 = __expb0G;
					break;
				case 0x3036: //Italian
					dvw = __dvbwI;
					dvp = __dvbpI;
					battle = __bbatI;
					party = __bparI;
					gen1 = true;
					expD1 = __expb1I;
					expD2 = __expb2I;
					expD0 = __expb0I;
					break;
				default:
					return;   /* If no match is found, exit function without doing anything */
				}
			break;
        case 0x00000000: // Red
			switch(__int) {
				case 0x3936: //English
					dvw = __dvrw;
					dvp = __dvrp;
					battle = __rbat;
					party = __rpar;
					gen1 = true;
					expD1 = __expr1;
					expD2 = __expr2;
					expD0 = __expr0;
					break;
				case 0x3039: //Japanese
					dvw = __dvrwJ;
					dvp = __dvrpJ;
					battle = __rbatJ;
					party = __rparJ;
					gen1 = true;
					expD1 = __expr1J;
					expD2 = __expr2J;
					expD0 = __expr0J;
					break;
				case 0x3035: //Spanish
					dvw = __dvrwS;
					dvp = __dvrpS;
					battle = __rbatS;
					party = __rparS;
					gen1 = true;
					expD1 = __expr1S;
					expD2 = __expr2S;
					expD0 = __expr0S;
					break;
				case 0x3033: //French
					dvw = __dvrwF;
					dvp = __dvrpF;
					battle = __rbatF;
					party = __rparF;
					gen1 = true;
					expD1 = __expr1F;
					expD2 = __expr2F;
					expD0 = __expr0F;
					break;
				case 0x3533: //German
					dvw = __dvrwG;
					dvp = __dvrpG;
					battle = __rbatG;
					party = __rparG;
					gen1 = true;
					expD1 = __expr1G;
					expD2 = __expr2G;
					expD0 = __expr0G;
					break;
				case 0x3935: //Italian
					dvw = __dvrwI;
					dvp = __dvrpI;
					battle = __rbatI;
					party = __rparI;
					gen1 = true;
					expD1 = __expr1I;
					expD2 = __expr2I;
					expD0 = __expr0I;
					break;
				default:
					return;   /* If no match is found, exit function without doing anything */
				}
			break;
        case 0x00574f4c: // Yellow
 			switch(__Yint) {
 				case 0x0000: //English
 					switch(__int) {
 						case 0x3830:
 							dvw = __dvyw;
 							dvp = __dvyp;
 							battle = __ybat;
 							party = __ypar;
 							gen1 = true;
							expD1 = __expy1;
							expD2 = __expy2;
							expD0 = __expy0;
 							break;
 					default:
 						return;   /* If no match is found, exit function without doing anything */
 					}
					break;
				case 0x3835: //Japanese
					dvw = __dvywJ;
					dvp = __dvypJ;
					battle = __ybatJ;
					party = __yparJ;
					gen1 = true;
					expD1 = __expy1J;
					expD2 = __expy2J;
					expD0 = __expy0J;					
					break;
				case 0x3731: //Spanish
					dvw = __dvywS;
					dvp = __dvypS;
					battle = __ybatS;
					party = __yparS;
					gen1 = true;
					expD1 = __expy1S;
					expD2 = __expy2S;
					expD0 = __expy0S;
					break;
				case 0x3631: //French
					dvw = __dvywF;
					dvp = __dvypF;
					battle = __ybatF;
					party = __yparF;
					gen1 = true;
					expD1 = __expy1F;
					expD2 = __expy2F;
					expD0 = __expy0F;
					break;
				case 0x3531: //German
					dvw = __dvywG;
					dvp = __dvypG;
					battle = __ybatG;
					party = __yparG;
					gen1 = true;
					expD1 = __expy1G;
					expD2 = __expy2G;
					expD0 = __expy0G;
					break;
				case 0x3931: //Italian
					dvw = __dvywI;
					dvp = __dvypI;
					battle = __ybatI;
					party = __yparI;
					gen1 = true;
					expD1 = __expy1I;
					expD2 = __expy2I;
					expD0 = __expy0I;
					break;
				default:
					return;   /* If no match is found, exit function without doing anything */
				}
			break;
		case 0x4720: // Green
			dvw = __dvgw;
			dvp = __dvgp;
			battle = __gbat;
			party = __gpar;
            gen1 = true;
			expD1 = __expg1;
			expD2 = __expg2;
			expD0 = __expg0;
            break;
        case 0x45584141: // Silver
            dvw = __dvsw;
//            dvp = __dvsp;
            dve = __dvse;
            egga = __segg; 
            battle = __sbat;
            party = __spar;
	    menu = __menus;
	    ibat = __ibats;
            gen2 = true;
            pdv1 = __dvstest;
//            pdv2 = __dvstest;
//            pdv3 = __dvstest;
//            pdv4 = __dvstest;
//            pdv5 = __dvstest;
//            pdv6 = __dvstest;
	    raikou = __roamsr;
	    entei = __roamse;
	    suicune = __roamss;
			expd0 = __exp0;
			expd1 = __exp1;
			expd2 = __exp2;
//			expd01 = __exp01;
//			expd11 = __exp11;
//			expd21 = __exp21;
//			expd02 = __exp02;
//			expd12 = __exp12;
//			expd22 = __exp22;
//			expd03 = __exp03;
//			expd13 = __exp13;
//			expd23 = __exp23;
//			expd04 = __exp04;
//			expd14 = __exp14;
//			expd24 = __exp24;
//			expd05 = __exp05;
//			expd15 = __exp15;
//			expd25 = __exp25;
			roamMe = __roamsMe;
			roamMr = __roamsMr;
			roamMs = __roamsMs;
			playM = __playsM;
	
            break;
        case 0x45554141: // Gold
            dvw = __dvsw;
//            dvp = __dvsp;
            dve = __dvse;
            egga = __segg;
            battle = __sbat;
            party = __spar;
            menu = __menus;
	    ibat = __ibats;
            gen2 = true;
            pdv1 = __dvstest;
//            pdv2 = __dvsp2;
//            pdv3 = __dvsp3;
 //           pdv4 = __dvsp4;
//            pdv5 = __dvsp5;
 //           pdv6 = __dvsp6;
            raikou = __roamsr;
            entei = __roamse;
			expd0 = __exp0;
			expd1 = __exp1;
			expd2 = __exp2;
//			expd01 = __exp01;
//			expd11 = __exp11;
//			expd21 = __exp21;
//			expd02 = __exp02;
//			expd12 = __exp12;
//			expd22 = __exp22;
//			expd03 = __exp03;
//			expd13 = __exp13;
//			expd23 = __exp23;
//			expd04 = __exp04;
//			expd14 = __exp14;
//			expd24 = __exp24;
//			expd05 = __exp05;
//			expd15 = __exp15;
//			expd25 = __exp25;
			roamMe = __roamsMe;
			roamMr = __roamsMr;
			roamMs = __roamsMs;
			playM = __playsM;
			
            break;
        case 0x45545942: // Crystal
            dvw = __dvcw;
//            dvp = __dvcp;
            dve = __dvce;
            egga = __cegg;
            battle = __cbat;
            party = __cpar;
            menu = __menuc;
	    ibat = __ibatc;
            gen2 = true;
	    crystal = 1;
            pdv1 = __dvcp1;
            pdv2 = __dvcp2;
            pdv3 = __dvcp3;
            pdv4 = __dvcp4;
            pdv5 = __dvcp5;
            pdv6 = __dvcp6;
            raikou = __roamcr;
            entei = __roamce;
			expd0 = __cexp0;
			expd1 = __cexp1;
			expd2 = __cexp2;
			expd01 = __cexp01;
			expd11 = __cexp11;
			expd21 = __cexp21;
			expd02 = __cexp02;
			expd12 = __cexp12;
			expd22 = __cexp22;
			expd03 = __cexp03;
			expd13 = __cexp13;
			expd23 = __cexp23;
			expd04 = __cexp04;
			expd14 = __cexp14;
			expd24 = __cexp24;
			expd05 = __cexp05;
			expd15 = __cexp15;
			expd25 = __cexp25;
			roamMe = __roamcMe;
			roamMr = __roamcMr;
			
			playM = __playcM;
            break;
        case 0x53545942: // SPA Crystal
            dvw = __dvcwS;
//            dvp = __dvcp;
            dve = __dvceS;
            egga = __ceggS;
            battle = __cbatS;
            party = __cparS;
            menu = __menucS;
            ibat = __ibatcS;
            gen2 = true;
            crystal = 1;
            pdv1 = __dvcp1S;
            pdv2 = __dvcp2S;
            pdv3 = __dvcp3S;
            pdv4 = __dvcp4S;
            pdv5 = __dvcp5S;
            pdv6 = __dvcp6S;
            raikou = __roamcrS;
            entei = __roamceS;
                        expd0 = __cexp0S;
                        expd1 = __cexp1S;
                        expd2 = __cexp2S;
                        expd01 = __cexp01S;
                        expd11 = __cexp11S;
                        expd21 = __cexp21S;
                        expd02 = __cexp02S;
                        expd12 = __cexp12S;
                        expd22 = __cexp22S;
                        expd03 = __cexp03S;
                        expd13 = __cexp13S;
                        expd23 = __cexp23S;
                        expd04 = __cexp04S;
                        expd14 = __cexp14S;
                        expd24 = __cexp24S;
                        expd05 = __cexp05S;
                        expd15 = __cexp15S;
                        expd25 = __cexp25S;
                        roamMe = __roamcMeS;
                        roamMr = __roamcMrS;

                        playM = __playcMS;
            break;
        default:
            return;   /* If no match is found, exit function without doing anything */
    }

    unsigned int testDV1 = 0xAAAA, /* Universal Shiny */
                 testDV2 = 0xAA3A, /* Gen 2 Shiny */
                 testDV3 = 0xAAAF, /* Gen 7 Shiny */
                 testDV4 = 0xFFFF; /* Perfect DV, Not Shiny */


    if (wildOn || battle == 1 && Auto == 1) {
        dvCalc(dvw, 14, 9, 92, 99);
        OvDrawString("Wild Pokemon", 14+1, 9+3, WHITE);
		if (ibat != 0)  {
           OvDrawString("(!)", 13+75, 94+3, BLUE);
         }
    }
    if ((partyOn || party == 1) && !battle) {
	if (gen2) {
//	    switch(menu) {
//	        case 0x1:    
		 dvp = pdv1;
		 expD1 = expd1;
		 expD2 = expd2;
		 expD0 = expd0;	 
//                break;
 //               case 0x2:
//                 dvp = pdv2;
//				 expD1 = expd11;
//				 expD2 = expd21;
//				 expD0 = expd01;
  //              break;
    //            case 0x3:
      //           dvp = pdv3;
//				 expD1 = expd12;
//				 expD2 = expd22;
//				 expD0 = expd02;
  //              break;
    //            case 0x4:
//                 dvp = pdv4;
//				 expD1 = expd13;
//				 expD2 = expd23;
//				 expD0 = expd03;
  //              break;
    //            case 0x5:
      //           dvp = pdv5;
//				 expD1 = expd14;
//				 expD2 = expd24;
//				 expD0 = expd04;
 //               break;
//                case 0x6:
//                 dvp = pdv6;
//				 expD1 = expd15;
//				 expD2 = expd25;
//				 expD0 = expd05;
//                break;
//	    default:
//		 dvp = pdv1;
//		}
	  }
		unsigned int exp = ((expD2<<16) | (expD1<<8)| expD0) % 25;
         dvCalc(dvp, 214, 9, 92, 110);
        OvDrawString("Party Pokemon", 214+1, 9+3, WHITE);
		OvDrawString(natruegne7[exp], 214+1, 104+3, PURPLE);
		
		
    }

    if ((((playM == 0x180B && crystal == 1) || (playM == 0x160B && crystal == 0))  || eggOn) && gen2 && party != 1 && !battle) {
        dvCalc(dve, 114, 9, 92, 99);
        OvDrawString("Unclaimed Egg", 114+1, 9+3, WHITE);
	 if ((egga & 0b00100000) != 0b0) {
           OvDrawString("(!)", 113+75, 94+3, GREEN);
         }
    }
	if (playM == roamMr){
		Rcol = 1;
	}
	else if (playM == roamMe) {
		Rcol = 0;
	}
	else if (playM == roamMs){
		Rcol = 2;
	}
    if ((roamOn && gen2) && !battle) {
        dvCalc(dvr, 14, 119, 92, 99);
        OvDrawString("Roamer", 14+1, 119+3, WHITE);
		if ((playM == roamMr) || (playM == roamMe) || ((playM == roamMs) && (crystal == 0))) {
			OvDrawString("(!)", 14+75, 202+3, rCol[Rcol]);
	}
    }
    char buf[40];
}

/* Returns currently pressed key */
u32 getKey()
{
    return (*(vu32*)(IoBasePad) ^ 0xFFF) & 0xFFF;
}

/*
Checks current key, then checks if button has been released since last check.
If it has, check is cleared and the next button press is processed. Toggles
overlays on or off when pressing L & R
*/
void handleKey(void) {
    u32 curkey = getKey();
    if(togglePressed) {
        if(curkey == 0)
            togglePressed = false;
    } else {
        switch(curkey) {
            case BUTTON_L:
                wildOn = !wildOn;
                togglePressed = true;
                break;
            case BUTTON_R:
                partyOn = !partyOn;
                togglePressed = true;
                break;
            case BUTTON_Y:
                eggOn = !eggOn;
                togglePressed = true;
                break;
            case BUTTON_R+BUTTON_DL:
                roamOn = true;
                togglePressed = true;
				dvr = raikou;
                break;
            case BUTTON_R+BUTTON_DD:
                roamOn = true;
                togglePressed = true;
                dvr = entei;
                break;
            case BUTTON_R+BUTTON_DR:
                roamOn = true;
                togglePressed = true;
                if(crystal == 0) {
                  dvr = suicune;
                 } else { 
		    	roamOn = false;
			}
                break;
            case BUTTON_R+BUTTON_DU:
                roamOn = false;
                togglePressed = true;
                break;
            case BUTTON_L+BUTTON_DU:
                Auto = !Auto;
                togglePressed = true;
                break;
		
         }
    }
}

/*
Overlay Callback.
isBottom: 1 for bottom screen, 0 for top screen.
addr: writable cached framebuffer virtual address, should flush data cache after modifying.
addrB: right-eye framebuffer for top screen, undefined for bottom screen.
stride: framebuffer stride(pitch) in bytes, at least 240*bytes_per_pixel.
format: framebuffer format, see https://www.3dbrew.org/wiki/GPU/External_Registers for details.

return 0 on success. return 1 when nothing in framebuffer was modified.
*/

u32 overlayCallback(u32 isBottom, u32 addr, u32 addrB, u32 stride, u32 format)
{
    OvSettings(addr, addrB, stride, format, isBottom);
    handleKey();

    /* Only draws overlays on bottom screen */
    if (isBottom)
        drawWidget();
    return 0;
}

/*
Checks if n3DS or o3DS, then initializes key capture.
Calls Overlay Callback
*/
int main()
{
    initSharedFunc();
    initSrv();

    bool isNew3DS = false;
    APT_CheckNew3DS(&isNew3DS);

    if (isNew3DS)
        IoBasePad = 0xFFFC2000;
    else
        IoBasePad = 0xFFFC6000;

    plgRegisterCallback(CALLBACK_OVERLAY, (void*) overlayCallback, 0);
    return 0;
}
